==> schluessel-0.3.0-src.tar.gz: schluessel/src/net/morilib/lisp/array/ArrayToVector.java <==
/*
 * Copyright 2009-2010 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.lisp.array;

import java.util.ArrayList;
import java.util.List;

import net.morilib.lisp.ConsListBuilder;
import net.morilib.lisp.Datum;
import net.morilib.lisp.Environment;
import net.morilib.lisp.LispMessage;
import net.morilib.lisp.LispVector;
import net.morilib.lisp.subr.UnaryArgs;

/**
 *
 *
 * @author MORIGUCHI, Yuichiro 2011/05/29
 */
public class ArrayToVector extends UnaryArgs {

	//
	private static Datum tovec(LispArray a, int dp, int[] is) {
		if(is.length == dp) {
			return a.getFromArray(is);
		} else if(dp > 0) {
			ConsListBuilder b = new ConsListBuilder();

			is[dp] = a.startIndex(dp);
			for(; is[dp] < a.endIndex(dp); is[dp]++) {
				b.append(tovec(a, dp + 1, is));
			}
			return b.get();
		} else {
			List<Datum> l = new ArrayList<Datum>();

			is[dp] = a.startIndex(dp);
			for(; is[dp] < a.endIndex(dp); is[dp]++) {
				l.add(tovec(a, dp + 1, is));
			}
			return new LispVector(l);
		}
	}

	/**
	 * 
	 * @param a
	 * @return
	 */
	public static Datum toVector(LispArray a) {
		return tovec(a, 0, new int[a.rank()]);
	}

	/* (non-Javadoc)
	 * @see net.morilib.lisp.subr.UnaryArgs#execute(net.morilib.lisp.Datum, net.morilib.lisp.Environment, net.morilib.lisp.LispMessage)
	 */
	@Override
	protected Datum execute(Datum c1a, Environment env,
			LispMessage mesg) {
		if(c1a instanceof LispArray) {
			return toVector((LispArray)c1a);
		} else {
			throw mesg.getError("err.srfi25.require.array", c1a);
		}
	}

}

==> schluessel-0.4.3-src.tar.gz: schluessel/src/net/morilib/lisp/array/ArrayToVector.java <==
/*
 * Copyright 2009-2010 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.lisp.array;

import java.util.ArrayList;
import java.util.List;

import net.morilib.lisp.ConsListBuilder;
import net.morilib.lisp.Datum;
import net.morilib.lisp.Environment;
import net.morilib.lisp.LispMessage;
import net.morilib.lisp.LispVector;
import net.morilib.lisp.subr.UnaryArgs;

/**
 *
 *
 * @author MORIGUCHI, Yuichiro 2011/05/29
 */
public class ArrayToVector extends UnaryArgs {

	//
	private static Datum tovec(ILispArray a, int dp, int[] is) {
		if(is.length == dp) {
			return a.getFromArray(is);
		} else if(dp > 0) {
			ConsListBuilder b = new ConsListBuilder();

			is[dp] = a.startIndex(dp);
			for(; is[dp] < a.endIndex(dp); is[dp]++) {
				b.append(tovec(a, dp + 1, is));
			}
			return b.get();
		} else {
			List<Datum> l = new ArrayList<Datum>();

			is[dp] = a.startIndex(dp);
			for(; is[dp] < a.endIndex(dp); is[dp]++) {
				l.add(tovec(a, dp + 1, is));
			}
			return new LispVector(l);
		}
	}

	/**
	 * 
	 * @param a
	 * @return
	 */
	public static Datum toVector(ILispArray a) {
		return tovec(a, 0, new int[a.rank()]);
	}

	/* (non-Javadoc)
	 * @see net.morilib.lisp.subr.UnaryArgs#execute(net.morilib.lisp.Datum, net.morilib.lisp.Environment, net.morilib.lisp.LispMessage)
	 */
	@Override
	protected Datum execute(Datum c1a, Environment env,
			LispMessage mesg) {
		if(c1a instanceof ILispArray) {
			return toVector((ILispArray)c1a);
		} else {
			throw mesg.getError("err.srfi25.require.array", c1a);
		}
	}

}
